#ifndef __MOLECULE_H
#define __MOLECULE_H
#ifdef __cplusplus
extern "C" {
#endif

typedef struct {
	PyObject_HEAD
	double x;		/* atom X coordinate */
	double y;		/* atom Y coordinate */
	double z;		/* atom Z coordinate */
	double radius;		/* atom radius */
	char name[5];		/* atom name */
	PyObject *first_parent;	/* atom's first Parent */
	PyObject *second_parent;/* atom's second Parent */
	PyObject *third_parent;	/* atom's third Parent */
	double fp_distance;	/* distance of atom to it's first Parent */
	double sp_angle;		/* angle made by atom with first and second 
				   parents */
	double tp_torsion;	/* torsion angle between planes defined by 
				   atom, first parent, second parent AND 
				   first parent, second parent, third parent */
	double fp_distance_tmp;	/* distance of atom to it's first Parent */
	double sp_angle_tmp;		/* angle made by atom with first and second 
				   parents */
	double tp_torsion_tmp;	/* torsion angle between planes defined by 
				   atom, first parent, second parent AND 
				   first parent, second parent, third parent */
	int bsep0, bsep1;
	int resnum;
	double sasa;
	double sasarad;
	double sasapar;
} Atom3d;

/* C API Function */
#define Atom3d_Type_NUM 0

/* create an Atom3d */
#define Atom3d_Create_RET Atom3d *
#define Atom3d_Create_PROTO Py_PROTO((void))
#define Atom3d_Create_NUM 1

/* square of Euclidean Distance between two Atoms */
#define Atom3d_SqDistance_RET double
#define Atom3d_SqDistance_PROTO Py_PROTO((Atom3d *, Atom3d *))
#define Atom3d_SqDistance_NUM 2

/* Euclidean Distance between two Atoms */
#define Atom3d_Distance_RET double
#define Atom3d_Distance_PROTO Py_PROTO((Atom3d *, Atom3d *))
#define Atom3d_Distance_NUM 3

/* Angle in degrees between 3 atoms */
#define Atom3d_Angle_RET double
#define Atom3d_Angle_PROTO Py_PROTO((Atom3d *, Atom3d *, Atom3d *))
#define Atom3d_Angle_NUM 4

/* torsion angle in degrees between 4 atoms */
#define Atom3d_Torsion_RET double
#define Atom3d_Torsion_PROTO Py_PROTO((Atom3d *, Atom3d *, Atom3d *, Atom3d *))
#define Atom3d_Torsion_NUM 5

/* geometric center of a collection of atoms */
#define Atom3d_GeoCenter_RET void
#define Atom3d_GeoCenter_PROTO Py_PROTO((double *, double *, double *,\
	                                 int, PyObject *))
#define Atom3d_GeoCenter_NUM 6

/* internal to cartesian coordinates for an atom */
#define Atom3d_IntToCart_RET void
#define Atom3d_IntToCart_PROTO Py_PROTO((Atom3d *))
#define Atom3d_IntToCart_NUM 7

/* zmatrix to cartesian coordinates for a collection of atoms */
#define Atom3d_ZmatrixToCart_RET void
#define Atom3d_ZmatrixToCart_PROTO Py_PROTO((int, PyObject *, int, int))
#define Atom3d_ZmatrixToCart_NUM 8

/* radius of gyration */
#define Atom3d_RadiusOfGyration_RET double
#define Atom3d_RadiusOfGyration_PROTO Py_PROTO((int, PyObject *))
#define Atom3d_RadiusOfGyration_NUM 9

/* rotation about X axis */
#define Atom3d_RotateX_RET void
#define Atom3d_RotateX_PROTO Py_PROTO((Atom3d *, double))
#define Atom3d_RotateX_NUM 10

/* rotation about Y axis */
#define Atom3d_RotateY_RET void
#define Atom3d_RotateY_PROTO Py_PROTO((Atom3d *, double))
#define Atom3d_RotateY_NUM 11

/* rotation about Z axis */
#define Atom3d_RotateZ_RET void
#define Atom3d_RotateZ_PROTO Py_PROTO((Atom3d *, double))
#define Atom3d_RotateZ_NUM 12

/* rotation about a bond */
#define Atom3d_BondRotate_RET void
#define Atom3d_BondRotate_PROTO Py_PROTO((Atom3d *, Atom3d *, PyObject *, double))
#define Atom3d_BondRotate_NUM 13
/* total number of C API pointers */
#define ATOM3D_API_POINTERS 14

#ifdef _ATOM3D_MODULE
/* type declaration */
extern DL_IMPORT(PyTypeObject) Atom3dType;
#endif

#if defined(_ATOM3D_MODULE) || defined(_ATOM3D_STATIC)
/* type check */
#define Atom3d_Check(ob) ((ob)->ob_type == &Atom3dType)

/* C API function declarations */

extern Atom3d_Create_RET Atom3d_Create Atom3d_Create_PROTO;
extern Atom3d_SqDistance_RET Atom3d_SqDistance Atom3d_SqDistance_PROTO;
extern Atom3d_Distance_RET Atom3d_Distance Atom3d_Distance_PROTO;
extern Atom3d_Angle_RET Atom3d_Angle Atom3d_Angle_PROTO;
extern Atom3d_Torsion_RET Atom3d_Torsion Atom3d_Torsion_PROTO;
extern Atom3d_GeoCenter_RET Atom3d_GeoCenter Atom3d_GeoCenter_PROTO;
extern Atom3d_IntToCart_RET Atom3d_IntToCart Atom3d_IntToCart_PROTO;
extern Atom3d_ZmatrixToCart_RET Atom3d_ZmatrixToCart Atom3d_ZmatrixToCart_PROTO;
extern Atom3d_RadiusOfGyration_RET Atom3d_RadiusOfGyration Atom3d_RadiusOfGyration_PROTO;
extern Atom3d_RotateX_RET Atom3d_RotateX Atom3d_RotateX_PROTO;
extern Atom3d_RotateY_RET Atom3d_RotateX Atom3d_RotateY_PROTO;
extern Atom3d_RotateZ_RET Atom3d_RotateX Atom3d_RotateZ_PROTO;
extern Atom3d_BondRotate_RET Atom3d_BondRotate Atom3d_BondRotate_PROTO;

#else
static void **ATOM3D_API;

/* type check */
#define Atom3d_Check(op) \
   ((op)->ob_type == (PyTypeObject *)ATOM3D_API[Atom3d_Type_NUM])

/*C API function declarations */

#define Atom3d_Create \
  (*(Atom3d_Create_RET (*)Atom3d_Create_PROTO) \
   ATOM3D_API[Atom3d_Create_NUM])
#define Atom3d_SqDistance \
  (*(Atom3d_SqDistance_RET (*)Atom3d_SqDistance_PROTO) \
   ATOM3D_API[Atom3d_SqDistance_NUM])
#define Atom3d_Distance \
  (*(Atom3d_Distance_RET (*)Atom3d_Distance_PROTO) \
   ATOM3D_API[Atom3d_Distance_NUM])
#define Atom3d_Angle \
  (*(Atom3d_Angle_RET (*)Atom3d_Angle_PROTO) \
   ATOM3D_API[Atom3d_Angle_NUM])
#define Atom3d_Torsion \
  (*(Atom3d_Torsion_RET (*)Atom3d_Torsion_PROTO) \
   ATOM3D_API[Atom3d_Torsion_NUM])
#define Atom3d_GeoCenter \
  (*(Atom3d_GeoCenter_RET (*)Atom3d_GeoCenter_PROTO) \
   ATOM3D_API[Atom3d_GeoCenter_NUM])
#define Atom3d_IntToCart \
  (*(Atom3d_IntToCart_RET (*)Atom3d_IntToCart_PROTO) \
   ATOM3D_API[Atom3d_IntToCart_NUM])
#define Atom3d_ZmatrixToCart \
  (*(Atom3d_ZmatrixToCart_RET (*)Atom3d_ZmatrixToCart_PROTO) \
   ATOM3D_API[Atom3d_ZmatrixToCart_NUM])
#define Atom3d_RadiusOfGyration \
  (*(Atom3d_RadiusOfGyration_RET (*)Atom3d_RadiusOfGyration_PROTO) \
   ATOM3D_API[Atom3d_RadiusOfGyration_NUM])
#define Atom3d_RotateX \
  (*(Atom3d_RotateX_RET (*)Atom3d_RotateX_PROTO) \
   ATOM3D_API[Atom3d_RotateX_NUM])
#define Atom3d_RotateY \
  (*(Atom3d_RotateY_RET (*)Atom3d_RotateY_PROTO) \
   ATOM3D_API[Atom3d_RotateY_NUM])
#define Atom3d_RotateZ \
  (*(Atom3d_RotateZ_RET (*)Atom3d_RotateZ_PROTO) \
   ATOM3D_API[Atom3d_RotateZ_NUM])
#define Atom3d_BondRotate \
  (*(Atom3d_BondRotate_RET (*)Atom3d_BondRotate_PROTO) \
   ATOM3D_API[Atom3d_BondRotate_NUM])

#define import_atom3d() \
{ \
  PyObject *module = PyImport_ImportModule("Atom3dc");\
  if (module != NULL) { \
    PyObject *module_dict = PyModule_GetDict(module); \
    PyObject *c_api_object = PyDict_GetItemString(module_dict, "_C_API"); \
    if (PyCObject_Check(c_api_object)) { \
      ATOM3D_API = (void **)PyCObject_AsVoidPtr(c_api_object); \
    } \
  } \
}
#endif

#ifdef __cplusplus
}
#endif
#endif











  
