def bump_check_intra(atoms, fai, res_id):

    i1 = fai[res_id]
    i2 = fai[res_id + 1]
    start = i1+5
    if start >= i2: return 0

    close = atoms[i1+3].close
    for i in range(i1+5, i2):
        if close(atoms[i]): return 1
    return 0

    
    
def bump_check(atoms, fai, ires1, ires2, jres1, jres2):

    for i in xrange(ires1, ires2):

        i1 = fai[i]
        i2 = fai[i+1]
        iCAClose = atoms[i1+1].close

        if jres1 > i:
            jrange = xrange(jres1, jres2)
        else:
            jrange = xrange(i+1, jres2)

        for j in jrange:
            j1 = fai[j]
            j2 = fai[j+1]
            if not iCAClose(atoms[j1+1], 10.0): continue
            for ia in xrange(i1, i2):
                atom = atoms[ia]
                Close = atom.close
                for ja in range(j1, j2):
                    jatom = atoms[ja]
                    if Close(jatom):
                        if j - i > 1: return 1
                        if atom.bsep1 + jatom.bsep0 > 3: return 1
            
    return 0

def print_all_bumps(protein, file):
    nr = protein.num_residues
    atoms = protein.atoms
    fai = protein.residue_first_atom_indices
    rn = protein.residue_names
    log = file.write
    nb = 0

    fmt = '%5d %3s %4s %5d %3s %4s Minimum = %g Actual = %g\n'

    for i in xrange(nr-1):
        i1 = fai[i]
        i2 = fai[i+1]
        iCAClose = atoms[i1+1].close
        for j in xrange(i+1, nr):
            j1 = fai[j]
            j2 = fai[j+1]
            if not iCAClose(atoms[j1+1], 8.0): continue
            for ia in xrange(i1, i2):
                atom = atoms[ia]
                Close = atom.close
                for ja in range(j1, j2):
                    jatom = atoms[ja]
                    if Close(jatom):
                        if j - i > 1:
                            nb = nb + 1
                            log(fmt % (i+1, rn[i], atom.name, j+1, rn[j],
                                       jatom.name, atom.radius+jatom.radius,
                                       atom.distance(jatom)))
                        elif atom.bsep1 + jatom.bsep0 > 4:
                            nb = nb + 1
                            log(fmt % (i+1, rn[i], atom.name, j+1, rn[j],
                                       jatom.name, atom.radius+jatom.radius,
                                       atom.distance(jatom)))

    log('\n Total Number of Bumps = %d\n' % nb)
    return nb
            
    
try:
    from linusc import bump_check, bump_check_intra
except ImportError:
    print 'Using slower python functions for bump_checking'
