RESIDUES = ['ALA', 'ARG', 'ASN', 'ASP', 'CYS', 'CYX', 'GLN', 'GLU',
            'GLX', 'GLY', 'HIS', 'HIP', 'ILE', 'LEU', 'LYS', 'MET',
            'PHE', 'PRO', 'SER', 'THR', 'TRP', 'TYR', 'VAL', 'ACE',
            'NME', 'PCA', 'FOR', 'ASX', 'AMD']
ONE_LETTER_CODES = {
    'ALA': 'A', 'ARG': 'R', 'ASN': 'N', 'ASP': 'D', 'CYS': 'C',
    'GLN': 'Q', 'GLU': 'E', 'GLY': 'G', 'HIS': 'H', 'ILE': 'I',
    'LEU': 'L', 'MET': 'M', 'PHE': 'F', 'PRO': 'P', 'SER': 'S',
    'THR': 'T', 'TRP': 'W', 'TYR': 'Y', 'VAL': 'V', 'LYS': 'K'}

_CHI1_DEFAULT = ('N', 'CA', 'CB', 'CG')
CHI1_ATOMS = {
    'ARG': _CHI1_DEFAULT,
    'ASN': _CHI1_DEFAULT,
    'ASP': _CHI1_DEFAULT,
    'CYS': ('N', 'CA', 'CB', 'SG'),
    'SER': ('N', 'CA', 'CB', 'OG'),
    'GLN': _CHI1_DEFAULT,
    'GLU': _CHI1_DEFAULT,
    'HIS': _CHI1_DEFAULT,
    'ILE': ('N', 'CA', 'CB', 'CG1'),
    'LEU': _CHI1_DEFAULT,
    'LYS': _CHI1_DEFAULT,
    'MET': _CHI1_DEFAULT,
    'PHE': _CHI1_DEFAULT,
    'PRO': _CHI1_DEFAULT,
    'THR': ('N', 'CA', 'CB', 'OG1'),
    'TRP': _CHI1_DEFAULT,
    'TYR': _CHI1_DEFAULT,
    'VAL': _CHI1_DEFAULT,
    'GLX': _CHI1_DEFAULT,
    'ASX': _CHI1_DEFAULT,
    'CYX': _CHI1_DEFAULT,
    'PCA': _CHI1_DEFAULT
    }

CHI2_ATOMS = {
    'ARG': ('CA', 'CB', 'CG', 'CD'),
    'ASN': ('CA', 'CB', 'CG', 'OD1'),
    'ASP': ('CA', 'CB', 'CG', 'OD1'),
    'ASX': ('CA', 'CB', 'CG', 'OD1'),
    'GLN': ('CA', 'CB', 'CG', 'CD'),
    'GLU': ('CA', 'CB', 'CG', 'CD'),
    'GLX': ('CA', 'CB', 'CG', 'CD'),
    'HIS': ('CA', 'CB', 'CG', 'ND1'),
    'ILE': ('CA', 'CB', 'CG1', 'CD1'),
    'LEU': ('CA', 'CB', 'CG', 'CD1'),
    'LYS': ('CA', 'CB', 'CG', 'CD'),
    'MET': ('CA', 'CB', 'CG', 'SD'),
    'PHE': ('CA', 'CD', 'CG', 'CD1'),
    'PRO': ('CA', 'CB', 'CG', 'CD'),
    'TRP': ('CA', 'CB', 'CG', 'CD1'),
    'TYR': ('CA', 'CB', 'CG', 'CD1')
    }

CHI3_ATOMS = {
    'ARG': ('CB', 'CG', 'CD', 'NE'),
    'GLN': ('CB', 'CG', 'CD', 'OE1'),
    'GLU': ('CB', 'CG', 'CD', 'OE1'),
    'GLX': ('CB', 'CG', 'CD', 'OE1'),
    'LYS': ('CB', 'CG', 'CD', 'CE'),
    'MET': ('CB', 'CG', 'SD', 'CE')
    }

CHI4_ATOMS = {
    'ARG': ('CG', 'CD', 'NE', 'CZ'),
    'LYS': ('CG', 'CD', 'CE', 'NZ')
    }

SASARAD = {
    'ALA': {'N':1.7, 'CA':2.0, 'C':1.7, 'O':1.4, 'CB':2.0,'OXT': 1.5},
    'ARG': {'N':1.7, 'CA':2.0, 'C':1.7, 'O':1.4, 'CB':2.0,
            'CG':2.0, 'CD': 2.0, 'NE': 1.7, 'CZ': 1.7,
            'NH1': 1.8, 'NH2': 1.8,'OXT':1.5},
    'ASN': {'N':1.7, 'CA':2.0, 'C':1.7, 'O':1.4, 'CB':2.0,
            'CG':1.5, 'OD1': 1.4, 'ND2':1.8,'OXT':1.5},
    'ASP': {'N':1.7, 'CA':2.0, 'C':1.7, 'O':1.4, 'CB':2.0,
            'CG':1.7, 'OD1': 1.5, 'OD2':1.5,'OXT':1.5},
    'CYS': {'N':1.7, 'CA':2.0, 'C':1.7, 'O':1.4, 'CB':2.0,
            'SG': 1.8,'OXT':1.5},
    'GLN': {'N':1.7, 'CA':2.0, 'C':1.7, 'O':1.4, 'CB':2.0,
            'CG':2.0, 'CD':1.7, 'OE1':1.4, 'NE2': 1.8,'OXT':1.5},
    'GLU': {'N':1.7, 'CA':2.0, 'C':1.7, 'O':1.4, 'CB':2.0,
            'CG':2.0, 'CD':1.7, 'OE1':1.5, 'OE2': 1.5,'OXT':1.5},
    'GLY': {'N':1.7, 'CA':2.0, 'C':1.7, 'O':1.4,'OXT':1.5},
    'HIS': {'N':1.7, 'CA':2.0, 'C':1.7, 'O':1.4, 'CB':2.0,
            'CG':1.7, 'ND1':1.5, 'CD2':1.85, 'CE1': 1.85,
            'NE2':1.7,'OXT':1.5},
    'ILE': {'N':1.7, 'CA':2.0, 'C':1.7, 'O':1.4, 'CB':2.0,
            'CG1': 2.0, 'CG2': 2.0, 'CD1': 2.0,'OXT':1.5},
    'LEU': {'N':1.7, 'CA':2.0, 'C':1.7, 'O':1.4, 'CB':2.0,
            'CG': 2.0, 'CD2': 2.0, 'CD1': 2.0,'OXT':1.5},
    'LYS': {'N':1.7, 'CA':2.0, 'C':1.7, 'O':1.4, 'CB':2.0,
            'CG': 2.0, 'CD': 2.0, 'CE': 2.0, 'NZ': 2.0,'OXT':1.5},
    'MET': {'N':1.7, 'CA':2.0, 'C':1.7, 'O':1.4, 'CB':2.0,
            'CG': 2.0, 'SD': 1.85, 'CE': 2.0,'OXT':1.5},
    'PHE': {'N':1.7, 'CA':2.0, 'C':1.7, 'O':1.4, 'CB':2.0,
            'CG': 1.7, 'CD1': 1.85, 'CD2': 1.85, 'CE1': 1.85,
            'CE2': 1.85, 'CZ': 1.85,'OXT':1.5},
    'PRO': {'N':1.7, 'CA':2.0, 'C':1.7, 'O':1.4, 'CB':2.0,
            'CG': 2.0, 'CD': 2.0,'OXT':1.5},
    'SER': {'N':1.7, 'CA':2.0, 'C':1.7, 'O':1.4, 'CB':2.0,
            'OG': 1.6,'OXT':1.5},
    'THR': {'N':1.7, 'CA':2.0, 'C':1.7, 'O':1.4, 'CB':2.0,
            'OG1': 1.6, 'CG2': 2.0,'OXT':1.5},
    'TRP': {'N':1.7, 'CA':2.0, 'C':1.7, 'O':1.4, 'CB':2.0,
            'CG': 1.7, 'CD1':1.85,'CD2':1.7,'NE1':1.7,
            'CE2':1.7,'CE3':1.85,'CZ2':1.85,'CZ3':1.85,
            'CH2':1.85,'OXT':1.5},
    'TYR': {'N':1.7, 'CA':2.0, 'C':1.7, 'O':1.4, 'CB':2.0,
            'CG':1.7, 'CD1':1.85, 'CD2':1.85, 'CE1':1.85,
            'CE2':1.85, 'CZ':1.7, 'OH':1.6,'OXT':1.5},
    'VAL': {'N':1.7, 'CA':2.0, 'C':1.7, 'O':1.4, 'CB':2.0,
            'CG1':2.0, 'CG2':2.0,'OXT':1.5},
    'ASX': {'N':1.7, 'CA':2.0, 'C':1.7, 'O':1.4, 'CB':2.0,
            'CG':1.7, 'AD1': 1.5, 'AD2':1.65,'OXT':1.5, 'OD1':1.5,
            'ND2': 1.7},
    'CYX': {'N':1.7, 'CA':2.0, 'C':1.7, 'O':1.4, 'CB':2.0,
            'SG': 1.8,'OXT':1.5},
    'GLX': {'N':1.7, 'CA':2.0, 'C':1.7, 'O':1.4, 'CB':2.0,
            'CG':2.0, 'CD':1.7, 'AE1':1.5, 'AE2':1.65,'OXT':1.5,
            'OE1':1.5,'NE2': 1.5},
    'ACE': {'CH3': 2.0, 'CA': 2.0, 'C':1.7, 'O': 1.4},
    'FOR': {'C': 1.7, 'O': 1.4},
    'NME': {'N': 1.7, 'CA': 2.0, 'CH3': 2.0},
    'AMD': {'N': 1.8},
    'PCA': {'N':1.70, 'CA':2.00, 'C':1.70, 'O':1.40, 'CB':2.00,
            'CG':2.00, 'CD':1.70,'OE':1.40}
    }

exposedTripeptideArea = {
        'ALA': {'N':   6.50,'CA':  13.60,'C':   1.10,'O':  25.00,'CB':  71.90},
        'CYS': {'N':   5.20,'CA':  10.50,'C':   1.30,'O':  25.60,'CB':  38.50,
                'SG':  65.00},
        'ASP': {'N':   5.10,'CA':  10.80,'C':   1.10,'O':  23.50,'CB':  33.50,
                'CG':   5.10,'OD1':  38.70,'OD2':  40.90},
        'GLU': {'N':   4.90,'CA':   9.20,'C':   1.00,'O':  22.70,'CB':  26.80,
                'CG':  33.20,'CD':   5.50,'OE1':  41.40,'OE2':  41.50},
        'PHE': {'N':   4.80,'CA':   9.20,'C':   1.20,'O':  23.20,'CB':  29.30,
                'CG':   0.60,'CD1':  22.10,'CD2':  36.70,'CE1':  37.70,
                'CE2':  36.10,'CZ':  21.90},
        'GLY': {'N':  11.00,'CA':  46.80,'C':   3.00,'O':  27.30},
        'HIS': {'N':   5.70,'CA':   9.40,'C':   1.40,'O':  23.90,'CB':  34.00,
                'CG':   1.60,'ND1':  11.40,'CD2':  51.60,'CE1':  30.50,
                'NE2':  33.00},
        'ILE': {'N':   3.60,'CA':   4.80,'C':   0.80,'O':  21.70,'CB':  11.00,
                'CG1':  37.00,'CG2':  58.40,'CD1':  43.70},
        'LYS': {'N':   5.50,'CA':   8.90,'C':   1.10,'O':  23.20,'CB':  25.90,
                'CG':  22.60,'CD':  28.00,'CE':  36.00,'NZ':  74.60},
        'LEU': {'N':   4.90,'CA':   5.90,'C':   1.10,'O':  23.40,'CB':  23.10,
                'CG':   9.60,'CD1':  63.40,'CD2':  61.70},
        'MET': {'N':   5.20,'CA':   8.20,'C':   1.10,'O':  24.10,'CB':  24.00,
                'CG':  29.60,'SD':  36.40,'CE':  74.80},
        'ASN': {'N':   5.50,'CA':  10.10,'C':   1.50,'O':  23.10,'CB':  32.10,
                'CG':   3.00,'OD1':  31.10,'ND2':  59.10},
        'PRO': {'N':   0.40,'CA':  13.20,'C':   0.60,'O':  21.40,'CB':  39.10,
                'CG':  44.30,'CD':  27.60},
        'GLN': {'N':   5.30,'CA':   8.20,'C':   0.90,'O':  23.40,'CB':  26.60,
                'CG':  30.90,'CD':   3.70,'OE1':  34.20,'NE2':  60.00},
        'ARG': {'N':   5.10,'CA':   9.40,'C':   1.20,'O':  23.40,'CB':  25.90,
                'CG':  23.90,'CD':  29.60,'NE':  13.30,'CZ':   2.20,
                'NH1':  58.60,'NH2':  63.40},
        'SER': {'N':   6.00,'CA':  12.30,'C':   1.20,'O':  24.50,'CB':  44.30,
                'OG':  41.50},
        'THR': {'N':   3.80,'CA':   9.00,'C':   1.10,'O':  24.00,'CB':  18.00,
                'OG1':  33.50,'CG2':  63.10},
        'VAL': {'N':   4.30,'CA':   7.30,'C':   1.00,'O':  23.50,'CB':  11.10,
                'CG1':  57.40,'CG2':  59.90},
        'TRP': {'N':   5.10,'CA':   7.30,'C':   1.20,'O':  23.80,'CB':  30.10,
                'CG':   1.30,'CD1':  30.20,'CD2':  29.80,'NE1':   3.30,
                'CE2':  38.50,'CE3':  37.90,'CZ2':  34.10,'CZ3':  21.50,
                'CH2':   2.20},
        'TYR': {'N':   4.80,'CA':   8.90,'C':   1.30,'O':  23.70,'CB':  29.20,
                'CG':   0.40,'CD1':  22.60,'CD2':  34.80,'CE1':   2.60,
                'CE2':  52.90,'CZ':  34.10,'OH':  21.50},
        'CYX': {'N':   5.20,'CA':  10.50,'C':   1.30,'O':  25.60,'CB':  38.50,
                'SG':  65.00},
        'ASX': {'N':   5.10,'CA':  10.80,'C':   1.10,'O':  23.50,'CB':  33.50,
                'CG':   5.10,'AD1':  38.70,'AD2':  40.90},
        'GLX': {'N':   4.90,'CA':   9.20,'C':   1.00,'O':  22.70,'CB':  26.80,
                'CG':  33.20,'CD':   5.50,'AE1':  41.40,'AE2':  41.50}
        }

buriedTripeptideArea = {
        'ALA': {'N':   1.30,'CA':   4.00,'C':   0.30,'O':   5.50,'CB':  20.90},
        'CYS': {'N':   0.50,'CA':   2.00,'C':   0.20,'O':   4.20,'CB':   3.30,
                'SG':   6.90},
        'ASP': {'N':   0.90,'CA':   3.70,'C':   0.30,'O':   5.40,'CB':  14.10,
                'CG':   2.30,'OD1':  11.20,'OD2':  24.80},
        'GLU': {'N':   0.90,'CA':   3.00,'C':   0.30,'O':   5.10,'CB':   9.80,
                'CG':  14.20,'CD':   3.10,'OE1':  14.40,'OE2':  27.60},
        'PHE': {'N':   0.60,'CA':   1.40,'C':   0.20,'O':   2.80,'CB':   3.30,
                'CG':   0.00,'CD1':   2.50,'CD2':   4.50,'CE1':   5.50,
                'CE2':   5.10,'CZ':   2.90},
        'GLY': {'N':   2.10,'CA':  17.00,'C':   0.80,'O':   7.60},
        'HIS': {'N':   0.80,'CA':   2.20,'C':   0.30,'O':   4.30,'CB':   6.90,
                'CG':   0.30,'ND1':   2.90,'CD2':  15.40,'CE1':  11.30,
                'NE2':  10.00},
        'ILE': {'N':   0.60,'CA':   1.00,'C':   0.20,'O':   2.70,'CB':   1.00,
                'CG1':   3.20,'CG2':   7.60,'CD1':   8.10},
        'LYS': {'N':   0.80,'CA':   3.20,'C':   0.30,'O':   6.00,'CB':   7.50,
                'CG':   9.90,'CD':  14.20,'CE':  20.70,'NZ':  49.80},
        'LEU': {'N':   0.60,'CA':   0.90,'C':   0.20,'O':   3.20,'CB':   2.60,
                'CG':   1.20,'CD1':   8.60,'CD2':   9.30},
        'MET': {'N':   1.00,'CA':   1.80,'C':   0.20,'O':   3.20,'CB':   3.90,
                'CG':   4.30,'SD':   5.60,'CE':  14.60},
        'ASN': {'N':   0.70,'CA':   2.80,'C':   0.30,'O':   6.00,'CB':  12.00,
                'CG':   1.30,'OD1':  14.30,'ND2':  25.10},
        'PRO': {'N':   0.10,'CA':   4.50,'C':   0.20,'O':   5.90,'CB':  16.70,
                'CG':  19.80,'CD':   9.50},
        'GLN': {'N':   0.60,'CA':   2.60,'C':   0.20,'O':   4.60,'CB':   7.30,
                'CG':  11.30,'CD':   1.80,'OE1':  17.80,'NE2':  25.90},
        'ARG': {'N':   1.00,'CA':   2.50,'C':   0.30,'O':   4.60,'CB':   6.40,
                'CG':   8.00,'CD':  12.00,'NE':   5.50,'CZ':   1.00,
                'NH1':  20.40,'NH2':  37.70},
        'SER': {'N':   1.20,'CA':   4.60,'C':   0.30,'O':   6.50,'CB':  18.80,
                'OG':  16.50},
        'THR': {'N':   0.70,'CA':   2.90,'C':   0.20,'O':   5.10,'CB':   5.10,
                'OG1':  11.90,'CG2':  23.60},
        'VAL': {'N':   0.60,'CA':   1.20,'C':   0.10,'O':   3.10,'CB':   1.50,
                'CG1':   9.60,'CG2':   8.50},
        'TRP': {'N':   0.30,'CA':   1.40,'C':   0.10,'O':   2.60,'CB':   3.30,
                'CG':   0.10,'CD1':   5.00,'CD2':   4.80,'NE1':   0.30,
                'CE2':   6.00,'CE3':   5.10,'CZ2':   3.60,'CZ3':   2.00,
                'CH2':   0.30},
        'TYR': {'N':   0.70,'CA':   1.70,'C':   0.30,'O':   4.30,'CB':   4.80,
                'CG':   0.10,'CD1':   3.90,'CD2':   7.80,'CE1':   0.60,
                'CE2':  17.70,'CZ':   8.50,'OH':   4.70},
        'CYX': {'N':   1.00,'CA':   1.40,'C':   0.40,'O':   4.90,'CB':   4.70,
                'SG':   3.90},
        'ASX': {'N':   0.90,'CA':   3.70,'C':   0.30,'O':   5.40,'CB':  14.10,
                'CG':   2.30,'AD1':  11.20,'AD2':  24.80},
        'GLX': {'N':   0.90,'CA':   3.00,'C':   0.30,'O':   5.10,'CB':   9.80,
                'CG':  14.20,'CD':   3.10,'AE1':  14.40,'AE2':  27.60}
        }

residueAreas = {
    "ALA": {"SS":  118.1, "Buried":   32.0},
    "ARG": {"SS":  256.0, "Buried":   99.4},
    "ASN": {"SS":  165.5, "Buried":   62.5},
    "ASP": {"SS":  158.7, "Buried":   62.7},
    "CYS": {"SS":  146.1, "Buried":   17.1},
    "GLN": {"SS":  193.2, "Buried":   72.1},
    "GLU": {"SS":  186.2, "Buried":   78.4},
    "GLY": {"SS":   88.1, "Buried":   27.5},
    "HIS": {"SS":  202.5, "Buried":   54.4},
    "ILE": {"SS":  181.0, "Buried":   24.4},
    "LEU": {"SS":  193.1, "Buried":   26.6},
    "LYS": {"SS":  225.8, "Buried":  112.4},
    "MET": {"SS":  203.4, "Buried":   34.6},
    "PHE": {"SS":  222.8, "Buried":   28.8},
    "PRO": {"SS":  146.6, "Buried":   56.7},
    "SER": {"SS":  129.8, "Buried":   47.9},
    "THR": {"SS":  152.5, "Buried":   49.5},
    "TRP": {"SS":  266.3, "Buried":   34.9},
    "TYR": {"SS":  236.8, "Buried":   55.1},
    "VAL": {"SS":  164.5, "Buried":   24.6}
}
