import Atom3d
import re
import string

RC_DICT = {
( 180, 180): 'A',
( 180,-120): 'B',
( 180, -60): 'C',
( 180,   0): 'D',
( 180,  60): 'E',
( 180, 120): 'F',
(-120, 180): 'G',
(-120,-120): 'H',
(-120, -60): 'I',
(-120,   0): 'J',
(-120,  60): 'K',
(-120, 120): 'L',
( -60, 180): 'M',
( -60,-120): 'N',
( -60, -60): 'O',
( -60,   0): 'P',
( -60,  60): 'Q',
( -60, 120): 'R',
(   0, 180): 'S',
(   0,-120): 'T',
(   0, -60): 'U',
(   0,   0): 'V',
(   0,  60): 'W',
(   0, 120): 'X',
(  60, 180): 'm',
(  60,-120): 'r',
(  60, -60): 'q',
(  60,   0): 'p',
(  60,  60): 'o',
(  60, 120): 'n',
( 120, 180): 'g',
( 120,-120): 'l',
( 120, -60): 'k',
( 120,   0): 'j',
( 120,  60): 'i',
( 120, 120): 'h'}

turns = ('OO', 'OP', 'OJ', 'PO', 'PP', 'PJ', 'JO', 'JP', 'JJ',
         'Mo', 'Mp', 'Mj', 'Ro', 'Rp', 'Rj',
         'oo', 'op', 'oj', 'po', 'pp', 'pj', 'jo', 'jp', 'jj',
         'mO', 'mP', 'mJ', 'rO', 'rP', 'rJ')
trndic = {}
for i in range(len(turns)):
    t = turns[i]
    if i < 9:
        trndic[t] = '1'
    elif i < 15:
        trndic[t] = '2'
    elif i < 24:
        trndic[t] = '3'
    else:
        trndic[t] = '4'

def res_rc(r1, r2, r3=180):
    if (abs(r3) <= 90.0):
        return '*'
    elif r1>180.0 or r2>180.0 or r3>180.0:
        return '?'
    ir1 = int(round(r1/60.0 )) * 60
    ir2 = int(round(r2/60.0 )) * 60

    if ir1 == -180: ir1 = 180
    if ir2 == -180: ir2 = 180
    return RC_DICT[(ir1,ir2)]

def rc_codes(chain, phi=None, psi=None, ome=None):
    n = range(len(chain))
    if phi is None: phi = map(chain.phi, n)
    if psi is None: psi = map(chain.psi, n)
    if ome is None: ome = map(chain.omega, n)
    return map(res_rc, phi, psi, ome)

def rc_ss(chain, phi=None, psi=None, ome=None):

    if phi is None:
        chain.gaps()
        nres = len(chain)
        phi = map(chain.phi, xrange(nres))
    else:
        nres = len(chain)
    if psi is None: psi = map(chain.psi, xrange(nres))
    if ome is None: ome = map(chain.omega, xrange(nres))

    codes = rc_codes(chain, phi, psi, ome)

    chain.gaps()

    sst = ['C']*nres

    is_turn = trndic.has_key

    for i in xrange(nres-1):
        code = codes[i] + codes[i+1]
        if is_turn(code):
            sst[i] = sst[i+1] = 'T'

    helices = _rc_find(codes, '[OP]', 5)
    strands = _rc_find(codes, '[LGFARM]', 3)

    for helix in helices:
        i, j = helix
        for k in range(i, j):
            sst[k] = 'H'
        for k in range(i-1, 0, -1):
            if codes[k] in 'OP':
                sst[k] = 'H'
            else:
                break
        for k in range(j, nres):
            if codes[k] in 'OP':
                sst[k] = 'H'
            else:
                break

    for strand in strands:
        i, j = strand
        for k in range(i, j):
            if sst[k] == 'C': sst[k] = 'E'


    return phi, psi, ome, sst
    
def _rc_find(codes, pattern, length=3):
    pattern_beg = re.compile(pattern + '{' + `length` + '}')
    pattern_end = re.compile(pattern + '(?!' +pattern +')')
    matches = []
    pos = 0
    if not type(codes) is type(''):
        codes = string.join(codes, '')
    while 1:
        res = pattern_beg.search(codes, pos)
        if res is None:
            break
        else:
            begin = res.regs[0][0]
            res = pattern_end.search(codes, res.regs[0][1]-1)     
            end = res.regs[0][1]
            matches.append((begin, end))
            pos = end + 1
    return matches

    
def add_backbone_hydrogens(chain):
    """Adds backbone polar hydrogens to a peptide chain"""

    res = chain[0]
            
    FROMINT = Atom3d.fromint
    has_h = 1
    for res in chain:
        if res.name <> 'PRO' and not res.has_atom('H'):
            has_h = 0
            break
    if has_h:
        return

    if res.name == 'PRO':
        try:
            n, ca, cd = res.atoms_with_name('N', 'CA', 'CD')
        except:
            pass
        else:
            h1 = FROMINT(n, 1.0, ca, 110.0, cd, 110.0)
            h2 = FROMINT(n, 1.0, ca, 110.0, cd,-110.0)
            h1.name = 'H1'
            h2.name = 'H2'
            res.elements.append(h1)
            res.elements.append(h2)
    else:
        try:
            n, ca, c = res.atoms_with_name('N', 'CA', 'C')
        except:
            pass
        else:
            h1 = FROMINT(n, 1.0, ca, 110.0, c, 180.0)
            h2 = FROMINT(n, 1.0, ca, 110.0, c, -60.0)
            h3 = FROMINT(n, 1.0, ca, 110.0, c,  60.0)
            h1.name = 'H1'
            h2.name = 'H2'
            h3.name = 'H3'
            res.elements.append(h1)
            res.elements.append(h2)
            res.elements.append(h3)

    residue_list = chain.elements
    for i in xrange(1, len(chain)):
        res = residue_list[i]
        prev = residue_list[i-1]
        if res.name == 'PRO' or hasattr(prev, 'gap'):
            continue
        else:
            try:
                c = prev.atoms_with_name('C')[0]
                o = prev.atoms_with_name('O')[0]
                n = res.atoms_with_name('N')[0]
            except:
                pass
            else:
                h = FROMINT(n, 1.0, c, 120.0, o, 180.0)
                h.name = 'H'
                res.elements.append(h)

def calc_hbonds(chain):
     don, acp = _get_don_acp_list(chain)
     hbdict = {}
     for h, n, i in don:
         hbdict[i] = []
         Append = hbdict[i].append
         hdisfunc = h.distance
         ndisfunc = n.distance
         for o, c, j in acp:
             if abs(j-i) <= 2: continue
             dho = hdisfunc(o)
             if dho > 4.0:
                 continue
             else:
                 e = _hbond_ene(dho, hdisfunc(c), ndisfunc(o), ndisfunc(c))
                 if e < -0.5: Append(e, j)
         hbdict[i].sort()
         del hbdict[i][2:]
     return hbdict

def dssp(chain, phi=None, psi=None, ome=None):


    if phi is None:
        chain.gaps()
        nres = len(chain)
        phi = map(chain.phi, xrange(nres))
    else:
        nres = len(chain)

    if psi is None: psi = map(chain.psi, xrange(nres))
    if ome is None: ome = map(chain.omega, xrange(nres))

    hbdict = calc_hbonds(chain)

    hb14 = [0]*nres
    hb13 = [0]*nres
    hb15 = [0]*nres
    sst = ['C']*nres

    keys = hbdict.keys()
    keys.sort()
        
    for i in range(nres):
        if hasattr(chain[i], 'gap'):
            sst[i] = '!'
            
    for key in keys:
        hbonds = hbdict[key]
        if hbonds:
            for e, j in hbonds:
                sep = key - j
                if sep == 4:
                    hb14[key] = e
                elif sep == 3:
                    hb13[key] = e
                elif sep == 5:
                    hb15[key] = e
    sst = _assign_helix(hb14, hb13, phi, psi, ome, sst)
    sst = _assign_turn(hb13, hb15, sst)

    for i in range(len(sst)):
        if sst[i] == 'H':
            try:
                del hbdict[i]
            except:
                pass
    sst = _assign_anti_parallel(hbdict, phi, psi, ome, sst)
    sst = _assign_parallel(hbdict, phi, psi, ome, sst)
    sst = _smooth_sheet(phi, psi, ome, sst)
    sst = _smooth_turns(phi, psi, ome, sst, hb14)
    return phi, psi, ome, sst


def _get_don_acp_list(chain):
    chain.add_backbone_hydrogens()
    nr = len(chain)
    donors = []
    acceptors = []

    res = chain[0]
    idx = res.idx
    try:
        n = res.atoms_with_name('N')[0]
    except:
        pass
    else:
        try:
            h1 = res.atoms_with_name('H1')[0]
        except:
            pass
        else:
            donors.append((h1, n, 0))
            try:
                h2 = res.atoms_with_name('H2')[0]
            except:
                pass
            else:
                donors.append((h2, n, 0))
                try:
                    h3 = res.atoms_with_name('H3')[0]
                except:
                    pass
                else:
                    donors.append((h3, n, 0))

    try:
        o, c = res.atoms_with_name('O', 'C')
    except:
        pass
    else:
        acceptors.append((o, c, 0))
            
    for i in range(1, len(chain)):
        res = chain[i]
        try:
            h, n = res.atoms_with_name('H', 'N')
        except:
            pass
        else:
            donors.append((h, n, i))

        try:
            o, c = res.atoms_with_name('O', 'C')
        except:
            pass
        else:
            acceptors.append((o, c, i))

    return donors, acceptors
    
def _hbond_ene(dho, dhc, dno, dnc, max = max):
    return max(27.888 * (1.0/dno + 1.0/dhc - 1.0/dho - 1.0/dnc), -9.9)

def _assign_helix(hb14, hb13, phi, psi, ome, sst):
    for i in range(4, len(sst)):
        if hb14[i] and hb14[i] < hb13[i]:
            if sst[i-2] == 'H':
                sst[i-1] = 'H'
            else:
                if _is_helix(phi[i-1], psi[i-1], ome[i-1]) and \
                   _is_helix(phi[i-2], psi[i-2], ome[i-2]) and \
                   _is_helix(phi[i-3], psi[i-3], ome[i-3]):
                    sst[i-3:i] = ['H', 'H', 'H']
                else:
                    sst[i-3:i] = ['T', 'T', 'T']
    return sst

def _is_helix(phi, psi, ome, abs=abs):
    if phi < -80 or phi > -40: return
    if psi < -60 or psi > -20: return
    if abs(ome) < 90: return
    return 1

def _assign_turn(hb13, hb15, sst):
    for i in xrange(len(sst)):
        if sst[i-1] == 'H':
            continue
        if hb13[i] < hb15[i]:
            sst[i-1] = 'T'
            sst[i-2] = 'T'
        elif hb15[i]:
            sst[i-1] = '5'
    return sst

def _assign_anti_parallel(hbdict, phi, psi, ome, sst):
    keys = hbdict.keys()
    keys.sort()
    for key in keys:
        hbonds = hbdict[key]
        if not hbonds: continue
        for e, j in hbonds:
            if hbdict.has_key(j):
                jhb = hbdict[j]
                if not jhb: continue
                for e2, k in jhb:
                    if k == key:
                        sst[key] = sst[k] = 'E'
    return sst
        
    
def _assign_parallel(hbdict, phi, psi, ome, sst):
    keys = hbdict.keys()
    keys.sort()
    for key in keys:
        hbonds = hbdict[key]
        if not hbonds: continue
        for e, j in hbonds:
            if hbdict.has_key(j+2):
                jhb = hbdict[j+2]
                if not jhb: continue
                for e2, k in jhb:
                    if k == key:
                        sst[key] = sst[k] = 'E'
    return sst
    
def _smooth_sheet(phi, psi, ome, sst):
    if sst[0] == 'C' and sst[1] == 'E':
        sst[0] = 'E'
    if sst[-1] == 'C' and sst[-2] == 'E':
        sst[-1] = 'E'
    for i in xrange(len(sst)-3):
        if sst[i] == 'C' and (sst[i-1] == 'E' or sst[i+1] == 'E') and \
           _is_strand(phi[i], psi[i], ome[i]):
            sst[i] = 'E'
    return sst

def _smooth_turns(phi, psi, ome, sst, hb14):
    for i in range(1, len(sst)-1):
        if sst[i] == 'T' and sst[i-1] == 'H':
            if _is_helix(phi[i], psi[i], ome[1]):
                sst[i] = 'H'
    turns = _rc_find(sst, 'T', 5)
    for i, j in turns:
        for r in range(i, j-2):
            if hb14[r+3] and _is_helix(phi[r], psi[r], ome[r]) and \
               _is_helix(phi[r+1], psi[r+1], ome[r+1]) and \
               _is_helix(phi[r+2], psi[r+2], ome[r+2]):
                sst[r:r+3] = ['H', 'H', 'H']
    for i in range(3, len(sst)-1):
        if sst[i] in '5T' and sst[i-1] == 'H' and \
           _is_helix(phi[i], psi[i], ome[i]):
            sst[i] = 'H'
    for i in range(len(sst)-1, 0, -1):
        if sst[i] in '5T' and sst[i+1] == 'H' and \
           _is_helix(phi[i], psi[i], ome[i]):
            sst[i] = 'H'
    return sst
    
    

def _is_strand(phi, psi, ome):
    if abs(ome) < 90: return 0
    if abs(phi)>=150 and abs(psi)>=150: return 1
    if -180<=phi<=-60 and 60<=psi<=180: return 1
    if 180>=phi>=60 and -60>=psi>=-180: return 1
    return 0

def przytyck_ss(chain, phi=None, psi=None, ome=None):
    chain.gaps()
    nres = range(len(chain))

    if phi is None: phi = map(chain.phi, nres)
    if psi is None: psi = map(chain.psi, nres)
    if ome is None: ome = map(chain.omega, nres)

    sst = ['X'] * len(nres)

    pos = 0    
    for f, s, w in map(None, phi, psi, ome):
        if hasattr(chain[pos], 'gap'):
            sst[pos] = '!'
        elif abs(w) <= 90 or f > 180 or s > 180:
            pass
        elif -80<=f<=-40 and -65<=s<=-5:
            sst[pos] = 'H'
        elif -110<=f<=-5 and -75<=s<=0:
            sst[pos] = 'h'
        elif -180<=f<=-60 and (80<=s<=180 or -180<=s<=-140):
            sst[pos] = 'E'
        elif f>=120 and (s>=120 or s<=-120):
            sst[pos] = 'E'
        pos = pos + 1

    for i in nres[1:]:
        if sst[i] == 'h' and (sst[i-1] == 'H' or sst[i+1] == 'H'):
            sst[i+1] = 'H'

    helices = _rc_find(sst, 'H', 5)
    strands = _rc_find(sst, 'E', 3)

    for i in nres:
        if sst[i] <> '!': sst[i] = 'C'

    codes = string.join(rc_codes(chain, phi, psi, ome), '')
    get_turn_type = trndic.get
    for i in nres[:-1]:
        if sst[i] == 'C':
            ttype = get_turn_type(codes[i:i+2], 0)
            if ttype:
                sst[i] = ttype
                sst[i+1] = ttype

    for i, j in helices:
        for k in range(i, j): sst[k] = 'H'
    for i, j in strands:
        for k in range(i, j): sst[k] = 'E'

    if nres[-1] > 1:
        if sst[1] == 'E':
            sst[0] = 'E'
        if sst[-2] == 'E':
            sst[-1] = 'E'


    return phi, psi, ome, sst


