import Rasmol
import string
import tempfile
import time
import os

class Viewer:
    def __init__(self, filename, format='pdb'):
        self.file = open(filename, 'r')
        self.tmpfile = tempfile.mktemp()
        self.unlink = os.unlink
        self.viewer = Rasmol.Rasmol()
        self.viewer_commands = ['wireframe 30']
        self.format = format

    def __del__(self):
##        self.closeViewer()
        self.file.close()
        self.unlink(self.tmpfile)

    def _write_tmp(self):
        tt = open(self.tmpfile, 'w')
        if self.format == 'xyz':
            line = self.file.readline()
            if not line:
                self.file.seek(0, 0)
                tt.close()
                raise EOFError, 'End of File Reached'
            else:
                n = string.atoi(line)
                tt.write(line)
                tt.write(self.file.readline())
                for i in range(n):
                    tt.write(self.file.readline())
                tt.close()
        else:
            while 1:
                line = self.file.readline()
                if not line:
                    self.file.seek(0, 0)
                    tt.close()
                    raise EOFError, 'End of File Reached'
                tt.write(line)
                if line[:3] in ('TER', 'END'):
                    break
                if line[:4] == 'ATOM':
                    continue
                elif line[:6] == 'COMPND':
                    c = string.strip(line[6:])
                    if c == '0':
                        self.color = 'red'
                    elif c == '1':
                        self.color = 'yellow'
                    else:
                        self.color = 'cpk'
        tt.close()

    def showNextConformation(self):
        self._write_tmp()
        self.viewer.load(self.tmpfile, self.format)
        self.viewer.send('color '+self.color)
        for command in self.viewer_commands:
            self.viewer.send(command)
        

    def animate(self, pause=1):
        while 1:
            try:
                self.showNextConformation()
            except EOFError:
                break
            else:
                time.sleep(pause)
        return 'break'

    def refresh(self):
        for command in self.viewer_commands:
            self.viewer.send(command)

    def closeViewer(self):
        os.path.unlink(self.tmpfile)
        self.viewer.quit()

    def addCommand(self, command):
        self.viewer_commands.append(command)


import Tkinter
import tkMessageBox
import tkSimpleDialog

class TkViewer:

    def __init__(self, parent, viewer):
        self.viewer = viewer
        self.frame = Tkinter.Frame(parent)
        self.nextButton = Tkinter.Button(self.frame, text='Next',
                                         command=self.showNext)
        self.nextButton.pack(side='left')
        self.animButton = Tkinter.Button(self.frame, text='Animate',
                                         command=self.showAnim)
        self.animButton.pack(side='left')
        self.quitButton = Tkinter.Button(self.frame, text='Quit',
                                         command=self.quit)
        self.quitButton.pack(side='left')
        self.frame.pack(side='top')
        self.optframe = Tkinter.Frame(parent)

        cb1 = Tkinter.IntVar()
        self.hbonds_cb = Tkinter.Checkbutton(self.optframe,
                                              text='Hbonds',
                                              variable = cb1,
                                              command = self._update_commands)
        self.hbonds_cb.pack(side='top', fill = 'both')
        self.hbonds_cb.var = cb1
        self.hbonds_cb.var.set(0)

        cb1 = Tkinter.IntVar()
        self.color_cb = Tkinter.Checkbutton(self.optframe,
                                              text='Color By Structure',
                                              variable = cb1,
                                              command = self._update_commands)
        self.color_cb.pack(side='top', fill = 'both')
        self.color_cb.pack(side='top', fill = 'both')
        self.color_cb.var = cb1
        self.color_cb.var.set(0)

        cb1 = Tkinter.IntVar()       
        self.cartoon_cb = Tkinter.Checkbutton(self.optframe,
                                              text='Cartoons',
                                              variable = cb1,
                                              command = self._update_commands)
        self.cartoon_cb.pack(side='top', fill = 'both')
        self.cartoon_cb.var = cb1
        self.cartoon_cb.var.set(0)

        self.optframe.pack(side='top')
        self.animation_pause_interval = 1
        self._update_commands()

    def showNext(self, e=None):
        try:
            self.viewer.showNextConformation()
        except EOFError:
            tkMessageBox.showerror('EOFError', 'End of File Reached')
        return 'break'

    def showAnim(self, e=None):
        self.animation_pause_interval = tkSimpleDialog.askfloat('Animation Speed',
                                        'Time between frames', 
                                        initialvalue=self.animation_pause_interval)
                  
        self.viewer.animate(self.animation_pause_interval)
        return 'break'

    def _update_commands(self, e=None):
        try:
            self.viewer.viewer_commands.remove('hbonds on')
        except ValueError:
            try:
                self.viewer.viewer_commands.remove('hbonds off')
            except ValueError:
                pass
        hbonds = self.hbonds_cb.var.get()
        if hbonds: self.viewer.addCommand('hbonds on')
        else: self.viewer.addCommand('hbonds off')

        try:
            self.viewer.viewer_commands.remove('color structure')
        except ValueError:
            try:
                self.viewer.viewer_commands.remove('color white')
            except ValueError:
                pass
        color = self.color_cb.var.get()
        if color: self.viewer.addCommand('color structure')
        #else: self.viewer.addCommand('color white')

        try:
            self.viewer.viewer_commands.remove('cartoon')
        except ValueError:
            try:
                self.viewer.viewer_commands.remove('cartoon off')
            except ValueError:
                pass
        cartoon = self.cartoon_cb.var.get()
        if cartoon: 
            self.viewer.addCommand('wireframe off')
            self.viewer.addCommand('cartoon')
        else: 
            self.viewer.addCommand('cartoon off') 
            self.viewer.addCommand('wireframe 30')
        self.viewer.refresh()

    def quit(self, e=None):
        del self.viewer
        raise SystemExit



if __name__ == "__main__":
    import sys, os
    file = sys.argv[1]
    try:
        format = sys.argv[2]
    except:
        format = 'pdb'
    root = Tkinter.Tk()
    viewer = Viewer(sys.argv[1], format)
    tkv = TkViewer(root, viewer)
    root.mainloop()


        
